/**********************************************************************************
 * CHARBUF charbuf.h
 * =================
 * versione    02
 * copyright   Calzoni Pietro aka Calzo
 * licenza     dual GPL/BSD
 * --------------------------------------------------------------------------------
 * File header di definizione di charbuf. Ora si vuole effettuare l'accesso al
 * device tramite /proc filesystem.
 *********************************************************************************/

#ifndef __CHARBUFFER_H__
#define __CHARBUFFER_H__

/// INCLUSIONI

#include <linux/init.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/kdev_t.h>
#include <linux/fs.h>
#include <linux/cdev.h>


/// DEFINIZIONI

#define MAX_BUF_DIMENSION 4096 //dimensione massima del buffer
#define DEF_BUF_DIMENSION 512  //dimensione di default del buffer

//#if define (DEF_BUF_DIMENSION > MAX_BUF_DIMENSION) //piccolo check
//# error "Error: DEF_BUF_DIMENSION > MAX_BUF_DIMENSION"
//#endif


#ifndef DEVICE_NAME
#define DEVICE_NAME "charbuf"
#endif

#ifndef CHARBUF_MAJOR
#define CHARBUF_MAJOR 249 //come scull
#endif

#ifndef CHARBUF_MINOR
#define CHARBUF_MINOR 0 //come scull
#endif

/// DICHIARAZIONI
/** charbuf_dev  la struttura principale che contiene il buffer per la scrittura.
 * si noti che il campo index  potenzialmente inutile se per esempio si delegasse
 * ad un carattere (x es '\0') la terminazione del buffer. L'introduzione di
 * `index' invece permette sia di usare un carattere in pi, sia di evitare lo scan
 * del buffer x trovare il limite. Ne consegue una pi elevata velocit */
struct charbuf_dev
{
   /// il buffer vero e proprio contenuto nel kernel
   char *charbuffer;
   ///dimensione netta del device
   unsigned int size;
   ///attuale posizione all'interno del buffer, necessario per l'append dei dati
   unsigned int index; 
   ///mutual exclusion semaphore - serve! vedi LDD3 credo cap3
   struct semaphore sem;
   ///credo sia necessario per la gestione dei dispositivi char
   struct cdev cdev;
};

/**********************************************
 * Funzioni di gestione
 * ********************************************/
int charbuf_menage_buffer(struct charbuf_dev *dev,int new_dimension);
int charbuf_init_buffer(struct charbuf_dev *dev,int dimension);


/// PROTOTIPI I/O

int charbuf_open(struct inode *inode,
		 struct file  *flip);

ssize_t charbuf_write(struct file *flip,
		      const char __user *buf, //__user: buf in user space = ATTENTO A QUELLO CHE SCRIVI
		      size_t count,
		      loff_t *f_pos);
  
ssize_t charbuf_read(struct file *flip,
		     char __user *buf,
		     size_t count,
		     loff_t *f_pos);

int charbuf_ioctl(struct inode *inode, struct file *filp,
		  unsigned int cmd, unsigned long arg);

#endif //__CHARBUFFER_H__
